from PyQt5 import QtCore, QtGui, QtWidgets

from PyQt5.QtCore import Qt
from .gui_fit import Ui_Dialog

from PyQt5.QtGui import QColor
from PyQt5.QtWidgets import QMessageBox, QLineEdit, QCheckBox
import numpy as np

import pyqtgraph as pg

from spectramanipulator import Logger, Spectrum

# from user_namespace import UserNamespace
from console import Console


# import lmfit
from lmfit import Minimizer
from .fitresult import FitResult

import fitmodels
import inspect


class FitDialog(QtWidgets.QDialog, Ui_Dialog):
    # static variables
    is_opened = False
    _instance = None

    # maximum number of parameters
    max_count = 20

    def __init__(self, spectrum, plot_widget, parent=None):
        super(FitDialog, self).__init__(parent, QtCore.Qt.WindowStaysOnTopHint)  # window stays on top
        self.setupUi(self)

        self.fitted_params = None
        self.covariance_matrix = None
        self.errors = None
        self.fitted_spectrum = None
        self.residual_spectrum = None

        self.plot_fit = None
        self.plot_residuals = None

        self.spectrum = spectrum
        self.plot_widget = plot_widget

        self.setWindowTitle("Fit of {}".format(self.spectrum.name))
        self.cbUpdateInitValues.setCheckState(Qt.Checked)

        f = 0.87
        x0, x1 = self.plot_widget.getViewBox().viewRange()[0]
        xy_dif = x1 - x0
        self.lr = pg.LinearRegionItem([x0 + (1 - f) * xy_dif, x0 + f * xy_dif],
                                      brush=QtGui.QBrush(QtGui.QColor(0, 255, 0, 20)),
                                      bounds=(self.spectrum.x_min(), self.spectrum.x_max()))
        self.lr.setZValue(-10)
        self.plot_widget.addItem(self.lr)

        self.leX0.returnPressed.connect(self.updateRegion)
        self.leX1.returnPressed.connect(self.updateRegion)

        self.lr.sigRegionChanged.connect(self.updatePlot)
        self.lr.sigRegionChangeFinished.connect(self.update_initial_values)
        self.updatePlot()

        self.sbParamsCount.valueChanged.connect(self.params_count_changed)
        self.btnFit.clicked.connect(self.fit)
        self.btnPrintReport.clicked.connect(self.print_report)
        self.btnPrintDiffEquations.clicked.connect(self.print_diff_eq)
        self.btnOK.clicked.connect(self.accept)
        self.btnCancel.clicked.connect(self.reject)
        self.cbModel.currentIndexChanged.connect(self.model_changed)

        self.sbParamsCount.setMaximum(self.max_count)

        self.params_list = []
        self.lower_bound_list = []
        self.value_list = []
        self.upper_bound_list = []
        self.fixed_list = []
        self.error_list = []

        for i in range(self.max_count):
            self.params_list.append(QLineEdit())
            self.lower_bound_list.append(QLineEdit())
            self.value_list.append(QLineEdit())
            self.upper_bound_list.append(QLineEdit())
            self.fixed_list.append(QCheckBox())
            self.error_list.append(QLineEdit())
            self.error_list[i].setReadOnly(True)

            self.gridLayout.addWidget(self.params_list[i], i + 1, 0, 1, 1)
            self.gridLayout.addWidget(self.lower_bound_list[i], i + 1, 1, 1, 1)
            self.gridLayout.addWidget(self.value_list[i], i + 1, 2, 1, 1)
            self.gridLayout.addWidget(self.upper_bound_list[i], i + 1, 3, 1, 1)
            self.gridLayout.addWidget(self.fixed_list[i], i + 1, 4, 1, 1)
            self.gridLayout.addWidget(self.error_list[i], i + 1, 5, 1, 1)

            self.fixed_list[i].stateChanged.connect(self.fixed_checked_changed)

        # get all models from fitmodels, get classes that inherits from Model base class and sort them by name
        # and number of species
        classes = inspect.getmembers(sys.modules[fitmodels.__name__], inspect.isclass)
        tuples = filter(lambda tup: issubclass(tup[1], fitmodels._Model) and tup[1] is not fitmodels._Model, classes)
        self.models = sorted(list(map(lambda tup: tup[1], tuples)), key=lambda cls: cls.name)
        # fill the available models combo box with model names
        self.cbModel.addItems(map(lambda m: m.name, self.models))

        self.methods = [
            {'name': 'Levenberg–Marquardt', 'abbr': 'leastsq'},
            {'name': 'Trust Region Reflective method', 'abbr': 'least_squares'},
            {'name': 'Differential evolution', 'abbr': 'differential_evolution'},
            {'name': 'Nelder-Mead, Simplex method (no error)', 'abbr': 'nelder'},
            {'name': 'L-BFGS-B (no error)', 'abbr': 'lbfgsb'},
            {'name': 'Powell (no error)', 'abbr': 'powell'}
        ]

        self.cbMethod.addItems(map(lambda m: m['name'], self.methods))

        self.cbCustom.stateChanged.connect(self.cbCustom_checked_changed)
        self.cbCustom_checked_changed()
        self.model_changed()

        self.current_model = None
        self.fit_result = None

        self.accepted = False
        FitDialog.is_opened = True
        FitDialog._instance = self

        self.show()
        self.exec()

    @staticmethod
    def get_instance():
        return FitDialog._instance

    @staticmethod
    def check_state(checked):
        return Qt.Checked if checked else 0

    def updateRegion(self):
        try:
            x0, x1 = float(self.leX0.text()), float(self.leX1.text())
            self.lr.setRegion((x0, x1))
        except ValueError:
            pass

    def updatePlot(self):
        x0, x1 = self.lr.getRegion()
        self.leX0.setText("{:.4g}".format(x0))
        self.leX1.setText("{:.4g}".format(x1))

    def cbCustom_checked_changed(self):
        if self.cbCustom.isChecked():
            self.sbParamsCount.setEnabled(True)
            self.pteEquation.setReadOnly(False)
            for i in range(self.max_count):
                self.params_list[i].setReadOnly(False)
        else:
            self.sbParamsCount.setEnabled(False)
            self.pteEquation.setReadOnly(True)
            for i in range(self.max_count):
                self.params_list[i].setReadOnly(True)

    def update_initial_values(self):

        if not self.cbUpdateInitValues.isChecked():
            return

        if self.current_model is None:
            return

        x0, x1 = self.lr.getRegion()

        start_idx = Spectrum.fi(self.spectrum.data[:, 0], x0)
        end_idx = Spectrum.fi(self.spectrum.data[:, 0], x1) + 1

        x_data = self.spectrum.data[start_idx:end_idx, 0]
        y_data = self.spectrum.data[start_idx:end_idx, 1]

        self.current_model.initialize_values(x_data, y_data)

        # iterate parameters and fill the values
        for i, p in enumerate(self.current_model.params.values()):
            self.value_list[i].setText("{:.4g}".format(p.value))

    def model_changed(self):
        # initialize new model
        self.current_model = self.models[self.cbModel.currentIndex()]()
        params_count = self.current_model.par_count()
        self.sbParamsCount.setValue(params_count)
        self.pteEquation.setPlainText(self.current_model.get_func_string())

        # iterate parameters and fill the fields
        for i, p in enumerate(self.current_model.params.values()):
            self.params_list[i].setText(p.name)
            self.lower_bound_list[i].setText(str(p.min))
            self.upper_bound_list[i].setText(str(p.max))
            self.fixed_list[i].setChecked(not p.vary)

        self.update_initial_values()

    def fit(self):
        try:

            self._fit()
        except Exception as e:
            Logger.message(e.__str__())
            QMessageBox.warning(self, 'Fitting Error', e.__str__(), QMessageBox.Ok)

    def remove_last_fit(self):
        if self.plot_fit is not None and self.plot_residuals is not None:
            self.plot_widget.plotItem.removeItem(self.plot_fit)
            self.plot_widget.plotItem.removeItem(self.plot_residuals)
            try:
                self.plot_widget.legend.removeLastItem()
                self.plot_widget.legend.removeLastItem()
            except IndexError:
                pass

    def _fit(self):
        import numpy as np
        from scipy.integrate import odeint

        x0, x1 = self.lr.getRegion()

        start_idx = Spectrum.fi(self.spectrum.data[:, 0], x0)
        end_idx = Spectrum.fi(self.spectrum.data[:, 0], x1) + 1

        x_data = self.spectrum.data[start_idx:end_idx, 0]
        y_data = self.spectrum.data[start_idx:end_idx, 1]

        # custom model check box checked, we have to create own model from scratch
        if self.cbCustom.isChecked():

            self.current_model = fitmodels._Model()

            p_names = []

            for i in range(int(self.sbParamsCount.value())):
                param = self.params_list[i].text()
                p_names.append(param)
                self.current_model.params.add(param)

            # create new function, fill params names after x and body of the function (take care of proper indentation)
            func_command = "def user_defined_func(x, {}):\n\t{}".format(','.join(p_names),
                self.pteEquation.toPlainText().replace('\n', '\n\t'))
            exec(func_command, locals(), globals())  # locals and globals had to be changed

            # change the blank function in abstract model by user defined
            self.current_model.func = user_defined_func

        if self.current_model is None:
            self.current_model = self.models[self.cbModel.currentIndex()]()

        # get params from field to current model
        for i in range(self.current_model.par_count()):
            param = self.params_list[i].text()
            self.current_model.params[param].value = float(self.value_list[i].text())
            self.current_model.params[param].min = float(self.lower_bound_list[i].text())
            self.current_model.params[param].max = float(self.upper_bound_list[i].text())
            self.current_model.params[param].vary = not self.fixed_list[i].isChecked()

        def residuals(params, x):
            return self.current_model.wrapper_func(x, params) - y_data

        method = self.methods[self.cbMethod.currentIndex()]['abbr']

        minimizer = Minimizer(residuals, self.current_model.params, fcn_args=(x_data,))
        result = minimizer.minimize(method=method)  # fit

        self.current_model.params = result.params

        # fill fields
        for i in range(self.current_model.par_count()):
            param = self.params_list[i].text()

            self.value_list[i].setText("{:.4g}".format(self.current_model.params[param].value))
            error = self.current_model.params[param].stderr
            self.error_list[i].setText("{:.4g}".format(error) if error is not None else '')

        y_fit_data = self.current_model.wrapper_func(x_data, self.current_model.params)
        y_residuals = y_data - y_fit_data

        self.remove_last_fit()

        self.plot_fit = self.plot_widget.plotItem.plot(x_data, y_fit_data,
                                                       pen=pg.mkPen(color=QColor(0, 0, 0, 200), width=2.5),
                                                       name="Fit of {}".format(self.spectrum.name))

        self.plot_residuals = self.plot_widget.plotItem.plot(x_data, y_residuals,
                                                             pen=pg.mkPen(color=QColor(255, 0, 0, 150), width=1),
                                                             name="Residuals of {}".format(self.spectrum.name))

        self.fitted_spectrum = Spectrum.from_xy_values(x_data, y_fit_data,
                                                       name="Fit of {}".format(self.spectrum.name),
                                                       color='black', line_width=2.5, line_type=Qt.SolidLine)

        self.residual_spectrum = Spectrum.from_xy_values(x_data, y_residuals,
                                                         name="Residuals of {}".format(self.spectrum.name),
                                                         color='red', line_width=1, line_type=Qt.SolidLine)

        values_errors = np.zeros((self.current_model.par_count(), 2), dtype=np.float64)

        for i in range(self.current_model.par_count()):
            param = self.params_list[i].text()
            values_errors[i, 0] = self.current_model.params[param].value
            error = self.current_model.params[param].stderr
            values_errors[i, 1] = error if error is not None else 0

        self.fit_result = FitResult(result, minimizer, values_errors,
                                    data_item=self.spectrum, fit_item=self.fitted_spectrum,
                                    residuals_item=self.residual_spectrum)

    def fixed_checked_changed(self, value):
        checkbox = self.sender()

        i = 0
        for i, ch in enumerate(self.fixed_list):
            if ch == checkbox:
                break

        self.lower_bound_list[i].setEnabled(True if value == Qt.Unchecked else False)
        self.upper_bound_list[i].setEnabled(True if value == Qt.Unchecked else False)

    def params_count_changed(self):

        count = int(self.sbParamsCount.value())

        for i in range(self.max_count):
            visible = count > i

            self.params_list[i].setVisible(visible)
            self.lower_bound_list[i].setVisible(visible)
            self.value_list[i].setVisible(visible)
            self.upper_bound_list[i].setVisible(visible)
            self.fixed_list[i].setVisible(visible)
            self.error_list[i].setVisible(visible)

    def set_result(self):
        pass

    # TODO-->>>
    def print_diff_eq(self):
        if self.current_model is None:
            return

        # self.current_model._print_diff_equations()

        # command = f"ax = plt.axes([0, 0, 0.1, 0.2])\nax.set_xticks([])\nax.set_yticks([])" \
        #     f"\nax.axis('off')\nplt.text(0.3, 0.4, \"${self.current_model._print_diff_equations()}$\", size=30)"
        #
        #
        # Console.execute_command(command)
        # #
        # Console.execute_command("plt\n%matplotlib inline")
        # self.current_model.print_diff_equations()

    def print_report(self):
        if self.fit_result is None:
            return
        try:
            if self.current_model is not None:
                report = "Fitting of '{}' with model '{}' succeeded. " \
                         "You can find fit results in variable " \
                         "'fit'.\n{}\n".format(self.spectrum.name, self.current_model.name,
                                               self.fit_result.report(print=False))

                Logger.console_message(report)
                Console.push_variables({'fit': self.fit_result})

        except Exception as e:
            Logger.message(e.__str__())
            QMessageBox.warning(self, 'Printing Error', e.__str__(), QMessageBox.Ok)

    def accept(self):
        self.remove_last_fit()
        self.print_report()

        self.accepted = True
        FitDialog.is_opened = False
        FitDialog._instance = None
        self.plot_widget.removeItem(self.lr)
        del self.lr
        super(FitDialog, self).accept()

    def reject(self):
        self.remove_last_fit()
        FitDialog.is_opened = False
        FitDialog._instance = None
        self.plot_widget.removeItem(self.lr)
        del self.lr
        super(FitDialog, self).reject()


if __name__ == "__main__":
    import sys


    def my_exception_hook(exctype, value, traceback):
        # Print the error and traceback
        print(exctype, value, traceback)
        # Call the normal Exception hook after
        sys._excepthook(exctype, value, traceback)
        sys.exit(1)


    from PyQt5.QtWidgets import QApplication

    sys._excepthook = sys.excepthook

    # Set the exception hook to our wrapping function
    sys.excepthook = my_exception_hook

    app = QtWidgets.QApplication(sys.argv)
    Dialog = FitDialog(None, None)
    # Dialog.show()
    sys.exit(app.exec_())



from PyQt5 import QtCore, QtGui, QtWidgets

from PyQt5.QtWidgets import QFileDialog, QMessageBox

from PyQt5.QtCore import Qt
from .gui_export_spectra_as import Ui_Dialog

from pathlib import Path

import os

from ..settings import Settings


class ExportSpectraAsDialog(QtWidgets.QDialog, Ui_Dialog):
    # static variables
    is_opened = False
    _instance = None

    def __init__(self, parent=None):
        super(ExportSpectraAsDialog, self).__init__(parent)
        self.setupUi(self)

        # disable resizing of the window,
        # help from https://stackoverflow.com/questions/16673074/in-qt-c-how-can-i-fully-disable-resizing-a-window-including-the-resize-icon-w
        self.setWindowFlags(Qt.Dialog | Qt.MSWindowsFixedSizeDialogHint)

        self.setWindowTitle("Export Selected Spectra As")

        self.leDir.setText(Settings.export_spectra_as_dialog_path)
        self.leFileExt.setText(Settings.export_spectra_as_dialog_ext)
        self.leDelimiter.setText(self.textualize_special_chars(Settings.export_spectra_as_dialog_delimiter))
        self.leDecimalSeparator.setText(Settings.export_spectra_as_dialog_decimal_sep)

        self.btnDir.clicked.connect(self.btnDir_clicked)

        self.accepted = False
        self.result = None

        ExportSpectraAsDialog.is_opened = True
        ExportSpectraAsDialog._instance = self

        self.show()
        self.exec()

    def btnDir_clicked(self):
        dir = QFileDialog.getExistingDirectory(self, "Select Directory", self.leDir.text())
        if dir != '':
            self.leDir.setText(dir)

    @staticmethod
    def get_instance():
        return ExportSpectraAsDialog._instance

    @staticmethod
    def textualize_special_chars(text):
        return text.replace('\n', '\\n').replace('\t', '\\t').replace('\r', '\\r')

    @staticmethod
    def DEtextualize_special_chars(text):
        return text.replace('\\n', '\n').replace('\\t', '\t').replace('\\r', '\r')

    def accept(self):

        if self.leDir.text() == '':
            QMessageBox.warning(self, 'Information', "Directory cannot be empty", QMessageBox.Ok)
            return

        try:
            path = Path(self.leDir.text())
            path.mkdir(parents=True, exist_ok=True)

            if not os.path.isdir(self.leDir.text()):
                raise Exception
        except Exception as ex:
            QMessageBox.warning(self, 'Information',
                                "Please check the directory path.\n{}".format(ex.__str__()),
                                QMessageBox.Ok)
            return

        if self.leDecimalSeparator.text() == '' or self.leFileExt.text() == '' or self.leDelimiter.text() == '':
            QMessageBox.warning(self, 'Information', "Fields cannot be empty.", QMessageBox.Ok)
            return

        ext = self.leFileExt.text()
        if not ext.startswith('.'):
            ext = '.' + ext

        delimiter = self.DEtextualize_special_chars(self.leDelimiter.text())

        self.result = (
            self.leDir.text(),
            ext.lower(),
            delimiter,
            self.leDecimalSeparator.text()
        )

        Settings.export_spectra_as_dialog_path = self.leDir.text()
        Settings.export_spectra_as_dialog_ext = ext
        Settings.export_spectra_as_dialog_delimiter = delimiter
        Settings.export_spectra_as_dialog_decimal_sep = self.leDecimalSeparator.text()

        Settings.save()

        self.accepted = True
        ExportSpectraAsDialog.is_opened = False
        ExportSpectraAsDialog._instance = None
        super(ExportSpectraAsDialog, self).accept()

    def reject(self):
        ExportSpectraAsDialog.is_opened = False
        ExportSpectraAsDialog._instance = None
        super(ExportSpectraAsDialog, self).reject()


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Dialog = ExportSpectraAsDialog()
    # Dialog.show()
    sys.exit(app.exec_())



from setuptools import setup, find_packages
from spectramanipulator import __version__

DESCRIPTION = 'Spectra Manipulator based on pyqtgraph'

setup(
    name="spectramanipulator",
    version=__version__,
    author="Dominik Madea",
    author_email="dominik.madea@gmail.com",
    url="https://github.com/dmadea/Spectra-Manipulator",
    download_url="https://github.com/dmadea/Spectra-Manipulator/archive/refs/tags/v0.1.1.tar.gz",
    description=DESCRIPTION,
    long_description=open('README.md').read(),
    packages=find_packages(),
    install_requires=[
        'lmfit',  # instals numpy and scipy
        'numpy',
        'scipy',
        'pyqtgraph>=0.12.1',
        'matplotlib',
        'pyqt5',
        'qtconsole>=5.1.0',
        'numba',
    ],

    keywords=['python', 'spectra manipulator'],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Education",
        "Programming Language :: Python :: 3",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English"
    ]
)


from PyQt5 import QtWidgets

from PyQt5.QtWidgets import QLineEdit, QMessageBox
from spectramanipulator.dialogs.genericinputdialog import GenericInputDialog
from PyQt5.QtCore import Qt


class ParamSettingsDialog(GenericInputDialog):

    def __init__(self, exp_dep_params: list, equal_pars: dict = None, set_result=None):

        if self.is_opened:
            return

        self.exp_dep_params = exp_dep_params
        self.equal_pars = equal_pars

        widget_list = []
        self.lineedit_list = []

        for par_dict in self.exp_dep_params:
            le = QLineEdit(', '.join(par_dict['values']))
            widget_list.append((f"{par_dict['name']}:", le))
            self.lineedit_list.append(le)

        widget_list.append(('Set equal parameter pairs (enter groups of experiment indexes encapsulated in round brackets, use comma as delimiter):', None))
        self.pars_line_edits = []

        for i, par_name in enumerate(map(lambda d: d['name'], self.exp_dep_params)):
            text = ''
            if par_name in self.equal_pars:
                text = ', '.join([f"({', '.join(str(idx) for idx in pair)})" for pair in self.equal_pars[par_name]])

            le = QLineEdit(text)
            self.pars_line_edits.append(le)
            widget_list.append((f'{par_name}:', le))

        super(ParamSettingsDialog, self).__init__(widget_list=widget_list,
                                                  label_text='Set all experiment parameters, separate values by comma:',
                                                  title='Settings of experiment-dependent parameters',
                                                  parent=None,
                                                  set_result=set_result,
                                                  flags=Qt.WindowStaysOnTopHint)

    def accept(self):
        self.equal_pars = {}
        for par_dict, le, le_equal_par in zip(self.exp_dep_params, self.lineedit_list, self.pars_line_edits):
            split = list(filter(None, le.text().split(',')))
            n_exp = len(par_dict['values'])
            if len(split) != n_exp:
                QMessageBox.critical(self, "Error", "The number of values must match the number of experiments!", QMessageBox.Ok)
                return
            try:
                par_dict['values'] = list(map(lambda s: float(s.strip()), split))
            except ValueError:
                QMessageBox.critical(self, "Error", "The values cannot be parsed as float numbers!", QMessageBox.Ok)
                return

            par_name = par_dict['name']
            text = le_equal_par.text()

            # parse input pairs into list of tuples
            try:
                string = ''.join(filter(lambda d: not d.isspace(), list(text)))  # remove white space chars
                if string == '':
                    continue

                split_text = list(filter(None, string.split('),')))  # remove empty entries
                self.equal_pars[par_name] = []

                for pairs in split_text:
                    pairs = pairs.replace('(', '').replace(')', '')  # remove residual brackets
                    split = list(filter(None, pairs.split(',')))
                    idxs = []
                    for val in split:
                        int_val = int(val)
                        idxs.append(int_val)
                        if int_val >= n_exp or int_val < 0:  # index is < 0 or > num of experiments
                            raise ValueError(f"Index must be in range of (0, {n_exp}).")
                        # index must not repeat in pairs
                        if int_val in [idx for pair in self.equal_pars[par_name] for idx in pair]:
                            raise ValueError("Index must not repeat in pairs.")

                    self.equal_pars[par_name].append(idxs)

            except ValueError as e:
                QMessageBox.critical(self, "Error", e.__str__(), QMessageBox.Ok)
                return
            except:
                QMessageBox.critical(self, "Error", "The equal parameters are input incorrectly!", QMessageBox.Ok)
                return

        super(ParamSettingsDialog, self).accept()


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Dialog = ParamSettingsDialog()
    Dialog.show()
    sys.exit(app.exec_())




from PyQt5 import QtCore, QtGui, QtWidgets

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import *
import sys


class GenericInputWidget(QtWidgets.QWidget):

    instance = None

    def __init__(self, widget_list=None, label_text='Some shity descriptive text...', title='GenericInputDialog',
                 parent=None):
        super(GenericInputWidget, self).__init__(parent)

        GenericInputWidget.instance = self

        self.accepted = False

        self.button_box = QtWidgets.QDialogButtonBox(self)
        self.button_box.setOrientation(QtCore.Qt.Horizontal)
        self.button_box.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel | QtWidgets.QDialogButtonBox.Ok)

        self.button_box.accepted.connect(self.accept)
        self.button_box.rejected.connect(self.reject)

        self.VLayout = QVBoxLayout()
        # self.VLayout.setContentsMargins(0, 0, 0, 0)

        self.label = QLabel(label_text)
        self.label.setWordWrap(True)

        self.VLayout.addWidget(self.label)

        self.grid_layout = QtWidgets.QGridLayout()
        self.grid_layout.setSpacing(7)
        self.grid_layout.setContentsMargins(15, 0, 0, 0)

        if widget_list is None:
            widget_list = [("par1", QLineEdit("some fucking text")),
                           ("par2", QLineEdit("some another fucking text")),
                           ("fuck paramters", QCheckBox("more shity text"))]

        self.label_list = []
        self.widget_list = []

        i = 0
        for label, widget in widget_list:
            self.label_list.append(QLabel(label + ':'))
            self.widget_list.append(widget)

            self.grid_layout.addWidget(self.label_list[i], i, 0)
            self.grid_layout.addWidget(widget, i, 1)
            i += 1

        self.VLayout.addItem(self.grid_layout)
        self.VLayout.addWidget(self.button_box)

        self.setLayout(self.VLayout)

        self.widget_list[0].setFocus()
        if isinstance(self.widget_list[0], QLineEdit):
            self.widget_list[0].selectAll()


    # virtual method
    def set_result(self):
        pass

    def accept(self):
        self.set_result()
        self.accepted = True
        GenericInputWidget.instance = None

    def reject(self):
        GenericInputWidget.instance = None


# if __name__ == "__main__":
#     import sys
#
#     app = QtWidgets.QApplication(sys.argv)
#     Dialog = GenericInputDialog()
#     Dialog.show()
#     sys.exit(app.exec_())



import numpy as np
import matplotlib.pyplot as plt
from .spectrum import Spectrum


def calculate_phis(item, c_0_act, c_0_com, act_idxs, com_idxs, phi_act, wl_range=(240, 330), em_source=None):
    sp = item[act_idxs[0]][0]

    idx_0 = Spectrum.find_nearest_idx(sp.data[:, 0], wl_range[0])
    idx_1 = Spectrum.find_nearest_idx(sp.data[:, 0], wl_range[1]) + 1

    # hardcoded number of solutions, this is not general, for other use, please change this number or rewrite the code
    solutions = 81
    wls = idx_1 - idx_0  # number of wavelengths
    wls_mat = np.tile(sp.data[idx_0:idx_1, 0], (solutions, 1))  # define matrix of wavelengths (x values for plotting)
    matrix = np.zeros((solutions, wls))  # define matrix of results, dimensions: 27 x [number of wavelengths]

    # for each wavelength in defined range
    for i in range(wls):
        # get the current wavelength
        wl = sp.data[idx_0 + i, 0]

        # calculate the 27 values at this wavelength
        phis = calculate_phi(item, c_0_act, c_0_com, act_idxs, com_idxs, phi_act, wl=wl, em_source=em_source)

        # fill the i-th column of the matrix with the solution
        matrix[:, i] = np.asarray(phis)

    # plot the matrix, parameter s is size of the circle, alpha sets transparency
    plt.rcParams['figure.figsize'] = [8, 5]
    plt.scatter(wls_mat, matrix, s=30, alpha=0.1)
    plt.ylabel('$\Phi$')
    plt.xlabel('Wavelength / nm')
    plt.show()

    return wls_mat, matrix


def calculate_phi(item, c_0_act, c_0_com, act_idxs, com_idxs, phi_act, wl=285, em_source=None):
    # define list of results
    phis = []

    # for every actinometer set
    for act in act_idxs:
        # for every compound set
        for com in com_idxs:
            # perform calculation between current sets and add the results into list
            phis += calculate_set(item[act], item[com], c_0_act, c_0_com, phi_act, wl, em_source)

    return phis


def calculate_set(act_set, com_set, c_0_act, c_0_com, phi_act, wl=285, em_source=None):
    # assuming that the number spectra in actinometer and compound set is the same
    # thus, len(act_set) == len(com_set), ** both act_set and com_set are instances of SpectrumList!

    # number of items in set
    length = len(act_set)

    # find index in data array of actinometer and compound that corresponds to the choosed wavelength
    idx_act = Spectrum.find_nearest_idx(act_set[0].data[:, 0], wl)
    idx_com = Spectrum.find_nearest_idx(com_set[0].data[:, 0], wl)

    # set absorbances at time zero for both actinometer and compound at wavelength wl
    A0_act = act_set[0].data[idx_act, 1]
    A0_com = com_set[0].data[idx_com, 1]

    # calculate the ratio of c0 and A0 for actinometer and compound
    c0A0_act = c_0_act / A0_act
    c0A0_com = c_0_com / A0_com

    # define list of computed quantum yields
    phis = []

    # for each actinometer difference - index i
    for i in range(1, length):
        # for each compound difference - index j
        for j in range(1, length):
            # calculate change of absorbance between two measurements for act and com
            dA_act = act_set[i - 1].data[idx_act, 1] - act_set[i].data[idx_act, 1]
            dA_com = com_set[j - 1].data[idx_com, 1] - com_set[j].data[idx_com, 1]

            # calculate corresponding change in concentrations
            dc_act = c0A0_act * dA_act
            dc_com = c0A0_com * dA_com

            # calculate the changes in times, use the value in name of the spectrum
            dt_act = float(act_set[i - 1].name) - float(act_set[i].name)
            dt_com = float(com_set[j - 1].name) - float(com_set[j].name)

            # calculate the quantum yield
            phi_com = phi_act * dc_com * dt_act / (dc_act * dt_com)

            # if user provided emission source, calculate the correction
            if em_source is not None:
                # calculate the absorbed light intensity (we use absorption spectrum
                # at the begining of the interval - i-1
                abs_int_act = em_source * (1 - 10 ** (-act_set[i - 1]))
                abs_int_com = em_source * (1 - 10 ** (-com_set[j - 1]))

                # integrate those spectra
                x_abs_act = abs_int_act.integral()
                x_abs_com = abs_int_com.integral()

                # apply the correction
                phi_com *= x_abs_act / x_abs_com

            # add the result to the list
            phis.append(phi_com)

    return phis





from spectramanipulator.parsers.csvfileparser import CSVFileParser
from spectramanipulator.parsers.generalparser import GeneralParser
from spectramanipulator.parsers.dxfileparser import DXFileParser
from spectramanipulator.parsers.parser import Parser
from spectramanipulator.parsers.xmlspreadsheetparser import parse_XML_Spreadsheet

__all__ = ['Parser', 'GeneralParser', 'DXFileParser', 'CSVFileParser', 'parse_XML_Spreadsheet']




import numpy as np
from PyQt5.QtGui import QColor


def int_default_color_scheme(counter):
    colors = [
        (255, 0, 0, 255),  # red
        (0, 255, 0, 255),  # green
        (0, 0, 255, 255),  # blue
        (0, 0, 0, 255),  # black
        (255, 255, 0, 255),  # yellow
        (255, 0, 255, 255),  # magenta
        (0, 255, 255, 255),  # cyan
        (155, 155, 155, 255),  # gray
        (155, 0, 0, 255),  # dark red
        (0, 155, 0, 255),  # dark green
        (0, 0, 155, 255),  # dark blue
        (155, 155, 0, 255),  # dark yellow
        (155, 0, 155, 255),  # dark magenta
        (0, 155, 155, 255)  # dark cyan
    ]

    return QColor(*colors[counter % len(colors)])


def intColorGradient(index, hues, grad_mat, reversed=False):
    """User defined gradient"""

    ind = int(index) % hues

    pos = ((hues - ind) / hues) if reversed else (ind / hues)

    positions = grad_mat[:, 0].flatten()
    idx_pos = np.searchsorted(positions, pos, side="right")
    idx_pos -= 1 if idx_pos > 0 else 0
    idx_pos -= 1 if idx_pos == len(positions) - 1 else 0

    # position within the interval of colors
    x = (pos - positions[idx_pos]) / (positions[idx_pos + 1] - positions[idx_pos])

    # calculate the resulting color as a linear combination of colors in the interval
    color_vector = (1 - x) * grad_mat[idx_pos, 1:] + x * grad_mat[idx_pos + 1, 1:]  # RGBA

    color = QColor(*color_vector)
    # color.setAlpha(int(color_vector[-1]))
    return color


def intColor(index, hues=9, values=1, maxValue=255, minValue=150, maxHue=360, minHue=0, sat=255, alpha=255,
             reversed=False):
    """
    Creates a QColor from a single index. Useful for stepping through a predefined list of colors.

    The argument *index* determines which color from the set will be returned. All other arguments determine what the set of predefined colors will be

    Colors are chosen by cycling across hues while varying the value (brightness).
    By default, this selects from a list of 9 hues."""
    hues = int(hues)
    values = int(values)
    ind = int(index) % (hues * values)
    indh = ind % hues
    indv = ind // hues
    if values > 1:
        v = minValue + indv * ((maxValue - minValue) / (values - 1))
    else:
        v = maxValue

    if reversed:
        h = minHue + ((hues - indh - 1) * (maxHue - minHue)) / hues
    else:
        h = minHue + (indh * (maxHue - minHue)) / hues

    c = QColor()
    c.setHsv(h, sat, v)
    c.setAlpha(alpha)
    return c


# -*- coding: utf-8 -*-

# fcns addItem and plot were modified by adding an optional parameter plot_legend
# plot handled zValue kwarg

from pyqtgraph.graphicsItems.PlotDataItem import PlotDataItem
from pyqtgraph.graphicsItems.PlotItem import PlotItem as _PlotItem


class PlotItem(_PlotItem):

    def addItem(self, item, *args, **kargs):
        """
        Add a graphics item to the view box. 
        If the item has plot data (PlotDataItem, PlotCurveItem, ScatterPlotItem), it may
        be included in analysis performed by the PlotItem.
        """
        self.items.append(item)
        vbargs = {}
        if 'ignoreBounds' in kargs:
            vbargs['ignoreBounds'] = kargs['ignoreBounds']
        self.vb.addItem(item, *args, **vbargs)
        name = None
        if hasattr(item, 'implements') and item.implements('plotData'):
            name = item.name()
            self.dataItems.append(item)
            # self.plotChanged()

            params = kargs.get('params', {})
            self.itemMeta[item] = params
            # item.setMeta(params)
            self.curves.append(item)
            # self.addItem(c)

        if hasattr(item, 'setLogMode'):
            item.setLogMode(self.ctrl.logXCheck.isChecked(), self.ctrl.logYCheck.isChecked())

        if isinstance(item, PlotDataItem):
            ## configure curve for this plot
            (alpha, auto) = self.alphaState()
            item.setAlpha(alpha, auto)
            item.setFftMode(self.ctrl.fftCheck.isChecked())
            item.setDownsampling(*self.downsampleMode())
            item.setClipToView(self.clipToViewMode())
            item.setPointMode(self.pointMode())

            ## Hide older plots if needed
            self.updateDecimation()

            ## Add to average if needed
            self.updateParamList()
            if self.ctrl.averageGroup.isChecked() and 'skipAverage' not in kargs:
                self.addAvgCurve(item)

            # c.connect(c, QtCore.SIGNAL('plotChanged'), self.plotChanged)
            # item.sigPlotChanged.connect(self.plotChanged)
            # self.plotChanged()
        # name = kargs.get('name', getattr(item, 'opts', {}).get('name', None))
        plot_legend = kargs.get('plot_legend', True)  # added
        if name is not None and hasattr(self, 'legend') and self.legend is not None and plot_legend:
            self.legend.addItem(item, name=name)

    def plot(self, *args, **kargs):
        """
        Add and return a new plot.
        See :func:`PlotDataItem.__init__ <pyqtgraph.PlotDataItem.__init__>` for data arguments

        Extra allowed arguments are:
            clear    - clear all plots before displaying new data
            params   - meta-parameters to associate with this data
        """
        clear = kargs.get('clear', False)
        params = kargs.get('params', None)

        if clear:
            self.clear()

        item = PlotDataItem(*args, **kargs)

        if params is None:
            params = {}

        if 'zValue' in kargs:
            item.setZValue(kargs['zValue'])

        # plot legend option added
        self.addItem(item, params=params, plot_legend=kargs.get('plot_legend', True))

        return item

















# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'spectramanipulator/dialogs\gui_fit_widget.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(448, 662)
        self.gridLayout_7 = QtWidgets.QGridLayout(Form)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.tabWidget = QtWidgets.QTabWidget(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tabWidget.sizePolicy().hasHeightForWidth())
        self.tabWidget.setSizePolicy(sizePolicy)
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayout = QtWidgets.QGridLayout(self.tab)
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_2 = QtWidgets.QLabel(self.tab)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_4.addWidget(self.label_2)
        self.cbModel = QtWidgets.QComboBox(self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cbModel.sizePolicy().hasHeightForWidth())
        self.cbModel.setSizePolicy(sizePolicy)
        self.cbModel.setObjectName("cbModel")
        self.horizontalLayout_4.addWidget(self.cbModel)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_9 = QtWidgets.QLabel(self.tab)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_3.addWidget(self.label_9)
        self.sbSpeciesCount = QtWidgets.QSpinBox(self.tab)
        self.sbSpeciesCount.setMinimum(1)
        self.sbSpeciesCount.setMaximum(20)
        self.sbSpeciesCount.setObjectName("sbSpeciesCount")
        self.horizontalLayout_3.addWidget(self.sbSpeciesCount)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.gridLayout.addLayout(self.verticalLayout, 1, 0, 1, 1)
        self.scrollAreaPredefinedModel = QtWidgets.QScrollArea(self.tab)
        self.scrollAreaPredefinedModel.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollAreaPredefinedModel.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollAreaPredefinedModel.setWidgetResizable(True)
        self.scrollAreaPredefinedModel.setObjectName("scrollAreaPredefinedModel")
        self.scrollAreaPredefinedModelContent = QtWidgets.QWidget()
        self.scrollAreaPredefinedModelContent.setGeometry(QtCore.QRect(0, 0, 394, 291))
        self.scrollAreaPredefinedModelContent.setObjectName("scrollAreaPredefinedModelContent")
        self.scrollAreaPredefinedModel.setWidget(self.scrollAreaPredefinedModelContent)
        self.gridLayout.addWidget(self.scrollAreaPredefinedModel, 2, 0, 1, 1)
        self.tabWidget.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.tab_2)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_21 = QtWidgets.QLabel(self.tab_2)
        self.label_21.setObjectName("label_21")
        self.horizontalLayout_5.addWidget(self.label_21)
        self.cbGenModels = QtWidgets.QComboBox(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cbGenModels.sizePolicy().hasHeightForWidth())
        self.cbGenModels.setSizePolicy(sizePolicy)
        self.cbGenModels.setObjectName("cbGenModels")
        self.horizontalLayout_5.addWidget(self.cbGenModels)
        self.btnSaveCustomModel = QtWidgets.QPushButton(self.tab_2)
        self.btnSaveCustomModel.setObjectName("btnSaveCustomModel")
        self.horizontalLayout_5.addWidget(self.btnSaveCustomModel)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.label_22 = QtWidgets.QLabel(self.tab_2)
        self.label_22.setObjectName("label_22")
        self.verticalLayout_2.addWidget(self.label_22)
        self.pteScheme = QtWidgets.QPlainTextEdit(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.pteScheme.sizePolicy().hasHeightForWidth())
        self.pteScheme.setSizePolicy(sizePolicy)
        self.pteScheme.setMinimumSize(QtCore.QSize(0, 30))
        self.pteScheme.setMaximumSize(QtCore.QSize(16777215, 100))
        self.pteScheme.setPlainText("")
        self.pteScheme.setObjectName("pteScheme")
        self.verticalLayout_2.addWidget(self.pteScheme)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.btnBuildModel = QtWidgets.QPushButton(self.tab_2)
        self.btnBuildModel.setObjectName("btnBuildModel")
        self.horizontalLayout_7.addWidget(self.btnBuildModel)
        self.cbShowBackwardRates = QtWidgets.QCheckBox(self.tab_2)
        self.cbShowBackwardRates.setChecked(False)
        self.cbShowBackwardRates.setObjectName("cbShowBackwardRates")
        self.horizontalLayout_7.addWidget(self.cbShowBackwardRates)
        self.verticalLayout_2.addLayout(self.horizontalLayout_7)
        self.cbVarProAmps = QtWidgets.QCheckBox(self.tab_2)
        self.cbVarProAmps.setChecked(True)
        self.cbVarProAmps.setObjectName("cbVarProAmps")
        self.verticalLayout_2.addWidget(self.cbVarProAmps)
        self.cbVarProIntercept = QtWidgets.QCheckBox(self.tab_2)
        self.cbVarProIntercept.setChecked(True)
        self.cbVarProIntercept.setObjectName("cbVarProIntercept")
        self.verticalLayout_2.addWidget(self.cbVarProIntercept)
        self.scrollAreaCustomModel = QtWidgets.QScrollArea(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(4)
        sizePolicy.setHeightForWidth(self.scrollAreaCustomModel.sizePolicy().hasHeightForWidth())
        self.scrollAreaCustomModel.setSizePolicy(sizePolicy)
        self.scrollAreaCustomModel.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollAreaCustomModel.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollAreaCustomModel.setWidgetResizable(True)
        self.scrollAreaCustomModel.setObjectName("scrollAreaCustomModel")
        self.scrollAreaCustomModelContent = QtWidgets.QWidget()
        self.scrollAreaCustomModelContent.setGeometry(QtCore.QRect(0, 0, 392, 157))
        self.scrollAreaCustomModelContent.setObjectName("scrollAreaCustomModelContent")
        self.scrollAreaCustomModel.setWidget(self.scrollAreaCustomModelContent)
        self.verticalLayout_2.addWidget(self.scrollAreaCustomModel)
        self.gridLayout_3.addLayout(self.verticalLayout_2, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tab_2, "")
        self.verticalLayout_3.addWidget(self.tabWidget)
        self.label_10 = QtWidgets.QLabel(Form)
        self.label_10.setObjectName("label_10")
        self.verticalLayout_3.addWidget(self.label_10)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_11 = QtWidgets.QLabel(Form)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout.addWidget(self.label_11)
        self.leX0 = MyLineEdit(Form)
        self.leX0.setObjectName("leX0")
        self.horizontalLayout.addWidget(self.leX0)
        self.label_12 = QtWidgets.QLabel(Form)
        self.label_12.setObjectName("label_12")
        self.horizontalLayout.addWidget(self.label_12)
        self.leX1 = MyLineEdit(Form)
        self.leX1.setObjectName("leX1")
        self.horizontalLayout.addWidget(self.leX1)
        self.btnShowRegion = QtWidgets.QPushButton(Form)
        self.btnShowRegion.setCheckable(True)
        self.btnShowRegion.setChecked(False)
        self.btnShowRegion.setObjectName("btnShowRegion")
        self.horizontalLayout.addWidget(self.btnShowRegion)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.gridLayout_4 = QtWidgets.QGridLayout()
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.label_23 = QtWidgets.QLabel(Form)
        self.label_23.setObjectName("label_23")
        self.gridLayout_4.addWidget(self.label_23, 1, 0, 1, 1)
        self.cbMethod = QtWidgets.QComboBox(Form)
        self.cbMethod.setObjectName("cbMethod")
        self.gridLayout_4.addWidget(self.cbMethod, 0, 1, 1, 1)
        self.label_13 = QtWidgets.QLabel(Form)
        self.label_13.setObjectName("label_13")
        self.gridLayout_4.addWidget(self.label_13, 0, 0, 1, 1)
        self.tbAlgorithmSettings = QtWidgets.QToolButton(Form)
        self.tbAlgorithmSettings.setCheckable(False)
        self.tbAlgorithmSettings.setChecked(False)
        self.tbAlgorithmSettings.setObjectName("tbAlgorithmSettings")
        self.gridLayout_4.addWidget(self.tbAlgorithmSettings, 0, 2, 1, 1)
        self.cbResWeighting = QtWidgets.QComboBox(Form)
        self.cbResWeighting.setObjectName("cbResWeighting")
        self.gridLayout_4.addWidget(self.cbResWeighting, 1, 1, 1, 2)
        self.verticalLayout_3.addLayout(self.gridLayout_4)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.cbShowResiduals = QtWidgets.QCheckBox(Form)
        self.cbShowResiduals.setChecked(False)
        self.cbShowResiduals.setObjectName("cbShowResiduals")
        self.horizontalLayout_8.addWidget(self.cbShowResiduals)
        self.cbFitBlackColor = QtWidgets.QCheckBox(Form)
        self.cbFitBlackColor.setChecked(False)
        self.cbFitBlackColor.setObjectName("cbFitBlackColor")
        self.horizontalLayout_8.addWidget(self.cbFitBlackColor)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem)
        self.verticalLayout_3.addLayout(self.horizontalLayout_8)
        spacerItem1 = QtWidgets.QSpacerItem(20, 28, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Ignored)
        self.verticalLayout_3.addItem(spacerItem1)
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.btnPrintReport = QtWidgets.QPushButton(Form)
        self.btnPrintReport.setCheckable(False)
        self.btnPrintReport.setChecked(False)
        self.btnPrintReport.setAutoRepeat(False)
        self.btnPrintReport.setAutoDefault(False)
        self.btnPrintReport.setObjectName("btnPrintReport")
        self.gridLayout_2.addWidget(self.btnPrintReport, 1, 1, 1, 1)
        self.btnCancel = QtWidgets.QPushButton(Form)
        self.btnCancel.setAutoDefault(False)
        self.btnCancel.setObjectName("btnCancel")
        self.gridLayout_2.addWidget(self.btnCancel, 1, 4, 1, 1)
        self.btnSimulateModel = QtWidgets.QPushButton(Form)
        self.btnSimulateModel.setCheckable(False)
        self.btnSimulateModel.setChecked(False)
        self.btnSimulateModel.setAutoRepeat(False)
        self.btnSimulateModel.setAutoDefault(False)
        self.btnSimulateModel.setObjectName("btnSimulateModel")
        self.gridLayout_2.addWidget(self.btnSimulateModel, 0, 0, 1, 1)
        self.btnFit = QtWidgets.QPushButton(Form)
        self.btnFit.setCheckable(False)
        self.btnFit.setChecked(False)
        self.btnFit.setAutoRepeat(False)
        self.btnFit.setAutoDefault(False)
        self.btnFit.setObjectName("btnFit")
        self.gridLayout_2.addWidget(self.btnFit, 1, 0, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem2, 1, 2, 1, 1)
        self.btnOK = QtWidgets.QPushButton(Form)
        self.btnOK.setAutoDefault(False)
        self.btnOK.setObjectName("btnOK")
        self.gridLayout_2.addWidget(self.btnOK, 0, 4, 1, 1)
        self.btnClearPlot = QtWidgets.QPushButton(Form)
        self.btnClearPlot.setCheckable(False)
        self.btnClearPlot.setChecked(False)
        self.btnClearPlot.setAutoRepeat(False)
        self.btnClearPlot.setAutoDefault(False)
        self.btnClearPlot.setObjectName("btnClearPlot")
        self.gridLayout_2.addWidget(self.btnClearPlot, 0, 1, 1, 1)
        self.verticalLayout_3.addLayout(self.gridLayout_2)
        self.gridLayout_7.addLayout(self.verticalLayout_3, 0, 0, 1, 1)

        self.retranslateUi(Form)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.label_2.setText(_translate("Form", "Model:"))
        self.label_9.setText(_translate("Form", "Species"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("Form", "Predefined Model"))
        self.label_21.setText(_translate("Form", "Saved models:"))
        self.btnSaveCustomModel.setText(_translate("Form", "Save Model As"))
        self.label_22.setText(_translate("Form", "<html><head/><body><p><span style=\" font-weight:600;\">Reaction model:</span></p></body></html>"))
        self.btnBuildModel.setText(_translate("Form", "Build Model"))
        self.cbShowBackwardRates.setText(_translate("Form", "Non-zero backward rates"))
        self.cbVarProAmps.setText(_translate("Form", "Calculate amplitudes from data by OLS"))
        self.cbVarProIntercept.setText(_translate("Form", "Calculate intercept from data by OLS"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), _translate("Form", "Custom Kinetic Model"))
        self.label_10.setText(_translate("Form", "Global fit range:"))
        self.label_11.setText(_translate("Form", "from"))
        self.label_12.setText(_translate("Form", "to"))
        self.btnShowRegion.setText(_translate("Form", "Show region"))
        self.label_23.setText(_translate("Form", "<html><head/><body><p>Residual weighting</p></body></html>"))
        self.label_13.setText(_translate("Form", "<html><head/><body><p>Minimizing algorithm:</p></body></html>"))
        self.tbAlgorithmSettings.setText(_translate("Form", "..."))
        self.cbShowResiduals.setText(_translate("Form", "Show residuals"))
        self.cbFitBlackColor.setText(_translate("Form", "Fits in black color"))
        self.btnPrintReport.setText(_translate("Form", "Print report"))
        self.btnCancel.setText(_translate("Form", "Cancel"))
        self.btnSimulateModel.setText(_translate("Form", "Simulate model"))
        self.btnFit.setText(_translate("Form", "Fit"))
        self.btnOK.setText(_translate("Form", "Ok"))
        self.btnClearPlot.setText(_translate("Form", "Clear Plot"))
from spectramanipulator.dialogs.mylineedit import MyLineEdit


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())




from PyQt5 import QtWidgets

from PyQt5.QtCore import Qt
from .gui_rename_dialog import Ui_Dialog


class RenameDialog(QtWidgets.QDialog, Ui_Dialog):

    # static variables
    is_opened = False
    _instance = None

    int32_max = 2147483647

    def __init__(self, expression='', offset=0, c_mult_facotr=1,
                 last_rename_take_name_from_list=False,  parent=None):
        super(RenameDialog, self).__init__(parent)
        self.setupUi(self)
        self.result = (expression, offset, c_mult_facotr)
        self.list = last_rename_take_name_from_list

        #disable resizing of the window,
        # help from https://stackoverflow.com/questions/16673074/in-qt-c-how-can-i-fully-disable-resizing-a-window-including-the-resize-icon-w
        self.setWindowFlags(Qt.Dialog | Qt.MSWindowsFixedSizeDialogHint)

        self.setWindowTitle("Rename Items")

        self.leExpression.setText(expression)
        self.leCounterMulFactor.setText(str(c_mult_facotr))

        self.sbOffset.setValue(offset)
        self.sbOffset.setMinimum(0)
        self.sbOffset.setMaximum(self.int32_max)

        self.cbTakeNamesFromList.setCheckState(self.check_state(last_rename_take_name_from_list))

        self.accepted = False

        RenameDialog.is_opened = True
        RenameDialog._instance = self

        self.leExpression.setFocus()
        self.leExpression.selectAll()

        self.cbTakeNamesFromList.stateChanged.connect(self.cbTakeNamesFromList_check_changed)

        # perform change
        self.cbTakeNamesFromList_check_changed()

        self.show()
        self.exec()

    @staticmethod
    def get_instance():
        return RenameDialog._instance

    @staticmethod
    def check_state(checked):
        return Qt.Checked if checked else 0

    def set_result(self):

        if self.is_renaming_by_expression:
            self.result = (self.leExpression.text(), self.sbOffset.value(), self.leCounterMulFactor.text())
        else:
            self.list = self.leList.text()

    def cbTakeNamesFromList_check_changed(self):
        if self.cbTakeNamesFromList.checkState() == Qt.Checked:
            self.sbOffset.setEnabled(False)
            self.leExpression.setEnabled(False)
            self.leList.setEnabled(True)
            self.leCounterMulFactor.setEnabled(False)
            self.is_renaming_by_expression = False
        else:
            self.sbOffset.setEnabled(True)
            self.leExpression.setEnabled(True)
            self.leList.setEnabled(False)
            self.leCounterMulFactor.setEnabled(True)
            self.is_renaming_by_expression = True

    def accept(self):
        self.set_result()
        self.accepted = True
        RenameDialog.is_opened = False
        RenameDialog._instance = None
        super(RenameDialog, self).accept()

    def reject(self):
        RenameDialog.is_opened = False
        RenameDialog._instance = None
        super(RenameDialog, self).reject()


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Dialog = RenameDialog()
    # Dialog.show()
    sys.exit(app.exec_())